###########################################################################
#
# IxLoad.tcl
#
# This implements the IxLoad package for *nix machines. It requires
# a remote Windows box with IxLoad Client installed, and provides
# a bridge to control the IxLoad Client from Unix. It has all the
# features of running the script directly on the Windows Client boxe.
#
# Running a script from a Unix box requires a call to the ::IxLoad connect
# command prior to issuing other IxLoad commands.
#
# Otherwise 
#
###########################################################################
#
set packageName "IxLoad"
foreach path $::env(TCLLIBPATH) {
    set ::auto_path [linsert $::auto_path 0 $path]
}
proc setPackage {pkgName} {
	global packageName
	set packageName $pkgName
}

package require IxTclHal
source [file join $::env(IXL_libs) lib IxLoadConnectCommon.tcl]

###########################################################################
#
# Implementation of the ::IxLoad connect command/subcommand
#
# if already connected to a host, then the connection is auto disconnected
# before
# 
###########################################################################

set successivePingsFailed 0

proc pingHost {host} {
if {[catch {exec ping $host -c 1} result]} { set result 0 } 
if { [regexp "0% packet loss"  $result]} { return 1} else { return 0 }
}

proc heartbeat {host} {
    global successivePingsFailed 
    set pingResult [pingHost $host]
    
	if { $pingResult == 0} {
        incr successivePingsFailed
    } else {
	    set successivePingsFailed 0
	}
	
	if { $successivePingsFailed == 5} {
        puts "Remote connection to host $host is down."
	    exit 0
	} else {
	    after 5000 heartbeat $host
	}
}

proc ::IxLoadPrivate::connect {args} {
    global ixAppPluginManager
    variable HostIP
    variable ChannelId
    variable TempChannelId
    
    set port ""
    if {[llength $args] == 0 || [llength $args] > 2} {
        error "Usage: ::IxLoad connect remoteHostOrIp {port}"
    } else {
        if {[llength $args] == 2} {
          set port [lindex $args 1]
        }
    }
    set ixloadhost [lindex $args 0]
 
    if { $ChannelId != ""} {
        # already connected to host other than ixloadhost, auto disconnect
        catch { disconnect }
    }
    # connect to Ixia Tcl Server.
    set errMsg ""
    if { [ ::tclServer::connectToTclServer [lindex $ixloadhost 0] errMsg ] } {
        error "Failed to connect to Tcl Server: ${errMsg} Make sure IxOS Tcl Server is installed and running on the Windows host machine."
    }

    set HostIP $ixloadhost

    # get the ip and port at which we will be listening
    set ip [lindex [fconfigure $::ixTclSvrHandle -sockname] 0]
    if {$port == ""} {
        # pic a port automatically
        set port [::comm::comm config -port]
    } else {
        # use the specified port
        ::comm::comm config -port $port
    }
    set TempChannelId "$port $ip"
    puts "Channel Id = $TempChannelId"
    
    #
    # accept remote connections
    #
    ::comm::comm config -local 0
    #
    # Later, when a connection comes in, we really have a ChannelId, 
    # so move TempChannelId into ChannelId
    #
    ::comm::comm hook incoming {
        puts "Incoming"
        #if { $::IxLoadPrivate::ChannelId == "" } {
	#    set ::IxLoadPrivate::ChannelId "$remport $::IxLoadPrivate::HostIP"
        #}
    }

    if { [catch {::clientSend $::ixTclSvrHandle \
            { 
                package require registry;
                if { [ catch { set IXLOAD_INSTALL_ROOT [registry get {HKEY_LOCAL_MACHINE\Software\Ixia Communications\IxLoad\9.10.115.43\InstallInfo} HOMEDIR] } result ] } {
                    error "IxLoad Client not installed on server: $result"
                }
            } } result ] } \
    {
        ::ixDisconnectTclServer;
        error $result
    }

    # Launch remote IxLoad script that will start servicing the comm port
    if { [catch {::clientSend $::ixTclSvrHandle {source [file join $IXLOAD_INSTALL_ROOT TclScripts remoteScriptingService IxLoadLaunchScript.tcl]} } result] } {
        ::ixDisconnectTclServer
        error "Failed to launch IxLoad Remote Scripting Service: $result"
    }

    global packageName
    if { [catch {::clientSend $::ixTclSvrHandle "IxLoadLaunchScript $ip $port $packageName" } result] } {
        ::ixDisconnectTclServer
        error "Failed to launch IxLoad Remote Scripting Service: $result"
    }

    vwait ::IxLoadPrivate::ChannelId
    puts "Connected to remote service"
    # issue any remote IxLoad command to force checking for first-time
    # access permission. The remote IxLoad connect command is a noop.
    if { [catch { Send {IxLoad connect} } result2] } \
    {
        disconnect
        error $result2
    }
    # connected

    if { $packageName=="IxLoadCsv" } {
    	Send {teepee eval {os.chdir("..\\TclScripts\\remoteScriptingService")}}
    }
	
	# heartbeat for remote connection
	heartbeat $HostIP

    # (re)initialize the global ixAppPluginManager to match server's current copy.
    set ixAppPluginManager [Send {set ixAppPluginManager}]
    return $result
}

###########################################################################
#
# Implementation of the ::IxLoad disconnect command/subcommand
#
# Raises an error if not currently connected
#
###########################################################################
proc ::IxLoadPrivate::disconnect {} {
    variable ChannelId
    if { $ChannelId == "" } {
	error "Attempt to disconnect but not connected"
    }
    set commResult ""
    if { [catch {::comm::comm shutdown $ChannelId} result] } {
	set commResult $result
    }
    set tclResult ""
    if { [catch {::ixDisconnectTclServer} result] } {
	set tclResult $result
    }
    # reset connection status indicator
    set ChannelId ""
    set TempChannelId ""
    if { $commResult != "" } {
        error $commResult
    }
    if { $tclResult != "" } {
	error $tclResult
    }
}

###########################################################################
#
# Internal Array trace function. This is used to remote array accesses on 
# IxLoad classes (which is used to access class static members of
# IxLoad classes, e.g. constants). It caches the results so that
# repeated calls do not result in refetching the data. This means
# that only python class constants can be fetched in this manner.
# Variables require a cget or a method call.
#
###########################################################################
proc ::IxLoadPrivate::arrayTrace {arrayName index op} {
    if { $op == "r" } {
	if { [array get $arrayName $index] == "" } {
	    # not defined, populate from remote host
	    set result [Send [format {set %s(%s)} $arrayName $index ]]
	    array set $arrayName [list $index $result]
	}
    } else {
	error "$op not allowed on array $arrayName"
    }
}

source [file join $::env(IXL_libs) lib stackManagerStaticClasses.tcl]
source [file join $::env(IXL_libs) lib lfpClasses.tcl]
